/*
    cj_types.j

    Basic replacement of JASS2 types to their C analogs.
    Private version, recommended for usage in imported libraries etc

    This file is part of cJass standard library
     2009 ADOLF \\ cjass.xgm.ru
*/

#guard my_types

function ABU takes unit A, unit B returns real
    return Atan2(GetUnitY(B)-GetUnitY(A), GetUnitX(B)-GetUnitX(A))*bj_RADTODEG
endfunction

function ABUC takes unit A, real B_x, real B_y returns real
    return Atan2(B_y-GetUnitY(A), B_x-GetUnitX(A))*bj_RADTODEG
endfunction

function ABUC2 takes real A_x, real A_y, real B_x, real B_y returns real
    return Atan2(B_y - A_y, B_x - A_x) * bj_RADTODEG
endfunction

function DBU takes unit A, unit B returns real
real dx = GetUnitX(B) - GetUnitX(A)
real dy = GetUnitY(B) - GetUnitY(A)
    return SquareRoot((dx * dx) + (dy * dy))
endfunction

function DBC takes unit A, real B_x, real B_y returns real
real dx = B_x - GetUnitX(A)
real dy = B_y - GetUnitY(A)
    return SquareRoot((dx * dx) + (dy * dy))
endfunction

function DBC2 takes real A_x, real A_y, real B_x, real B_y returns real
real dx = B_x - A_x
real dy = B_y - A_y
    return SquareRoot((dx * dx) + (dy * dy))
endfunction

function DBC_Ex takes unit A, real B_x, real B_y returns real
real dx = B_x - GetUnitX(A)
real dy = B_y - GetUnitY(A)
    return GetMaxAvailableDistanceEx(A, SquareRoot((dx * dx) + (dy * dy)), ABUC(A, B_x, B_y))
endfunction

function TBU takes unit A, unit B, real speed returns real
real dx = GetUnitX(B) - GetUnitX(A)
real dy = GetUnitY(B) - GetUnitY(A)
    return (SquareRoot((dx * dx) + (dy * dy)) / speed)
endfunction

function TBC takes unit A, real B_x, real B_y, real speed returns real
real dx = B_x - GetUnitX(A)
real dy = B_y - GetUnitY(A)
    return (SquareRoot((dx * dx) + (dy * dy)) / speed)
endfunction

function TBC_Ex takes unit A, real B_x, real B_y, real speed returns real
real dx = B_x - GetUnitX(A)
real dy = B_y - GetUnitY(A)
    return (GetMaxAvailableDistanceEx(A, SquareRoot((dx * dx) + (dy * dy)), ABUC(A, B_x, B_y))/speed)
endfunction

function ParabolaZ takes real h, real d, real x returns real
  return (4 * h / d) * (d - x) * (x / d)
endfunction

function GetAngleOfRebound takes real current_angle, real angle_between_unit_and_bound, real range returns boolean
    return (-current_angle + 180. + 2. * angle_between_unit_and_bound) + GetRandomReal(-range, range)
endfunction

function IsUnitBack takes unit uF, unit uWhichBack returns boolean
real r1 = bj_RADTODEG * Atan2(GetUnitY(uWhichBack) - GetUnitY(uF), GetUnitX(uWhichBack) - GetUnitX(uF)) + 360.
real r2 = GetUnitFacing(uWhichBack) + 360.
   if GetUnitY(uWhichBack) < GetUnitY(uF) then
       set r1 = r1 + 360.
   endif
   return (r1<=(r2+45.) and r1>=(r2-45.))
endfunction

function IsSide takes unit ataker,unit victim returns boolean
real angle1 = GetUnitFacing(victim)
real angle2 = ABU(ataker,victim)

    if not (GetUnitY(victim) > GetUnitY(ataker)) {
        angle1 -= 360.
    }
    
    if (angle2 <= ( angle1 + 135.00 ) and angle2 >= ( angle1 + 45.00 ))\\
    or (angle2 <= ( angle1 - 45.00 ) and angle2 >= ( angle1 - 135.00 ))\\
    or (GetUnitY(victim) > GetUnitY(ataker) and GetUnitX(victim) > GetUnitX(ataker) and angle2 <= ( angle1 - 225.00 ) and angle2 >= ( angle1 - 315.00 ) )\\
    or (GetUnitY(victim) < GetUnitY(ataker) and GetUnitX(victim) > GetUnitX(ataker) and angle2 >= ( angle1 + 225.00 ) and angle2 <= ( angle1 + 315.00 ))
    {
        return true
    }
return false
endfunction

function Text takes string s, unit u, unit u2, real size,real r,real g, real b returns nothing
//real sp =(64 * 0.071) / 128
real x1 = 0.0355 * Cos(ABU(u,u2) * bj_DEGTORAD)
real y1 = 0.0355 * Sin(ABU(u,u2) * bj_DEGTORAD)
    bj_lastCreatedTextTag = CreateTextTag()
    SetTextTagText(bj_lastCreatedTextTag, s, (size * 0.023) / 10)
    //SetTextTagPos(bj_lastCreatedTextTag, x, y, 7)
    SetTextTagPosUnit(bj_lastCreatedTextTag, u2, 7)
    SetTextTagColor(bj_lastCreatedTextTag, PercentToInt(r, 255), PercentToInt(g, 255), PercentToInt(b,255), 0)
    SetTextTagVelocity(bj_lastCreatedTextTag, x1,y1)
    SetTextTagPermanent(bj_lastCreatedTextTag, false)
    SetTextTagLifespan(bj_lastCreatedTextTag, 2.5)
    SetTextTagFadepoint(bj_lastCreatedTextTag, 0)
endfunction

function AddSound takes string s, real x, real y returns nothing
sound snd = CreateSound(s, false, true, true, 10, 10, "CombatSoundsEAX")
    SetSoundChannel(snd, Channel)
    Channel++
    if Channel > 40 { Channel = 35 }
    SetSoundVolume(snd, 127)
    SetSoundPitch(snd, 1)
    SetSoundDistances(snd, 600, 10000)
    SetSoundDistanceCutoff(snd, 2100)
    SetSoundConeAngles(snd, 0.0, 0.0, 127)
    SetSoundConeOrientation(snd, 0.0, 0.0, 0.0)
    SetSoundPosition(snd, x, y, 50.)
    StartSound(snd)
    KillSoundWhenDone(snd)
snd = null
endfunction

function RandomFromGroup takes group g returns unit
    set bj_groupRandomConsidered = CountUnitsInGroup(g)
    call ForGroup(g, function RandomEnum)
    call GroupClear(bj_groupAddGroupDest)
    call DestroyGroup(bj_groupAddGroupDest)
return bj_groupRandomCurrentPick
endfunction

function DamageUnitsInRange takes unit source, real x, real y, real range, real damage, attacktype at, int filter_type returns nothing
unit victim
group g = CG
    if filter_type == ALL { 
        GroupEnumUnitsInRange(g, x, y, range, Filter(function AllFilter)) 
    }
    elseif filter_type == ENEMIES { 
        ForFilter1 = source 
        GroupEnumUnitsInRange(g, x, y, range, Filter(function EnemiesFilter)) 
    }
        while(FirstOfGroup(g) != null) {
            victim = FirstOfGroup(g)
            UnitDamageTarget(source, victim, damage, true, false, at, DAMAGE_TYPE_NORMAL, WEAPON_TYPE_WHOKNOWS)
            PainSound(victim, 10.)
            GroupRemoveUnit(g, victim)
        }
    GC(g)
    DG(g)
g = null
victim = null
endfunction
